'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _constants = require('./constants');

var isMac = function isMac() {
  return navigator.platform.toUpperCase().indexOf('MAC') >= 0;
};

/**
 * Return if the key combined with the modifier key(s) is printable.
 *
 * Examples:
 * - Windows: Alt+D does not print anything in input field
 * - macOS: Alt+D prints ∂
 *
 * Generally we find that:
 * - Windows: Alt and Ctrl are reserved for browser shortcuts and stops chars
 * from printing
 * - macOS: Command is reserved for browser shortcuts and stops chars from
 * printing
 *
 * @param {keyInfo} Information about the pressed key 
 */
var isPrintable = function isPrintable(keyInfo) {
  var isOneChar = keyInfo.keyName.length === 1;
  var hasBrowserShortcutKey = void 0;
  if (isMac()) {
    var meta = keyInfo.modifierKeys.indexOf(_constants.KEYS.meta) > -1;
    hasBrowserShortcutKey = meta;
  } else {
    var alt = keyInfo.modifierKeys.indexOf(_constants.KEYS.alt) > -1;
    var ctrl = keyInfo.modifierKeys.indexOf(_constants.KEYS.ctrl) > -1;
    hasBrowserShortcutKey = alt || ctrl;
  }

  // Older browsers use these for the 'key' element of KeyboardEvents for the numpad.
  var numpadKeys = [_constants.KEYS.numpadAdd, _constants.KEYS.numpadSubtract, _constants.KEYS.numpadMultiply, _constants.KEYS.numpadDivide];

  var isNumpad = numpadKeys.indexOf(keyInfo.keyName) > -1;
  isOneChar = isOneChar || isNumpad;

  return isOneChar && !hasBrowserShortcutKey;
};

/**
 * Returns an array of pressed modifier keys.
 *
 * The elements of the array are strings used by KeyboardEvent.key to represent
 * modifier keys:
 * https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/key/Key_Values
 *
 * @param {KeyboardEvent} The native event to inspect for modifier keys 
 */
var getPressedModifiers = function getPressedModifiers(nativeEvent) {
  var modifierKeys = [_constants.KEYS.meta, _constants.KEYS.ctrl, _constants.KEYS.shift, _constants.KEYS.alt];
  var pressedModifierKeys = modifierKeys.filter(function (key) {
    return nativeEvent[key];
  });
  return pressedModifierKeys;
};

var getHistoryKey = function getHistoryKey() {
  return isMac() ? _constants.KEYS.meta : _constants.KEYS.ctrl;
};

exports.default = { isPrintable: isPrintable, getPressedModifiers: getPressedModifiers, getHistoryKey: getHistoryKey };
module.exports = exports['default'];